<?php
/**
 * Utilisations de pipelines par Territoires.
 *
 * @package    SPIP\TERRITOIRES\PIPELINES
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Ajout de contenu sur certaines pages, notamment des formulaires de liaisons entre objets.
 * Territoires ajoute les éventuels caractéristiques complémentaires et le formulaire d'association.
 *
 * @pipeline affiche_milieu
 *
 * @param array $flux Données du pipeline
 *
 * @return array Données du pipeline modifiées pour refléter le traitement.
 */
function territoires_affiche_milieu(array $flux) : array {
	if (isset($flux['args']['exec'])) {
		$texte = '';
		$exec = trouver_objet_exec($flux['args']['exec']);

		if (
			$exec
			and ($exec['edition'] !== true) // page visu
			and ($id_table = $exec['id_table_objet'])
			and ($objet = $exec['type'])
			and isset($flux['args'][$id_table])
			and ($id_objet = (int) ($flux['args'][$id_table]))
		) {
			// Insertion des extras de type 'info' chargés pour ce type de territoire
			if ($objet === 'territoire') {
				// -- acquisition de l'objet
				include_spip('action/editer_objet');
				$territoire = objet_lire('territoire', $id_objet);
				// -- tester si les extras 'info' sont chargées pour le type de territoire
				include_spip('inc/territoires_unite_peuplement');
				$extra_peuple = unite_peuplement_extra_est_charge($territoire['type'], $territoire['iso_pays'], 'info');
				if ($extra_peuple) {
					$texte .= recuperer_fond(
						'prive/objets/liste/territoires_extras',
						[
							'iso_territoire' => $territoire['iso_territoire'],
							'type_extra'     => 'info',
							'titre'          => _T('territoires:titre_liste_extras_info')
						]
					);
				}
			}

			// Insertion du formulaire d'association
			if (
				($table = $exec['table_objet_sql'])
				and include_spip('inc/config')
				and (in_array($table, lire_config('territoires/association_objets', [])))
			) {
				$texte .= recuperer_fond(
					'prive/objets/editer/liens',
					[
						'table_source' => 'territoires',
						'objet'        => $objet,
						'id_objet'     => $id_objet,
						'editable'     => autoriser('associerterritoires', $objet, $id_objet) ? 'oui' : 'non'
					]
				);
			}

			// Insertion dans la fiche objet.
			if ($texte) {
				if ($p = strpos($flux['data'], '<!--affiche_milieu-->')) {
					$flux['data'] = substr_replace($flux['data'], $texte, $p, 0);
				} else {
					$flux['data'] .= $texte;
				}
			}
		}
	}

	return $flux;
}

/**
 * Ajout de contenu dans la colonne « gauche » des pages de l’espace privé, notamment « la fiche objet »
 * Territoires ajoute le bloc des codes alternatifs.
 *
 * @pipeline affiche_gauche
 *
 * @param array $flux Données du pipeline
 *
 * @return array Données du pipeline modifiées pour refléter le traitement.
 */
function territoires_affiche_gauche(array $flux) : array {
	if (isset($flux['args']['exec'])) {
		$texte = '';
		$exec = trouver_objet_exec($flux['args']['exec']);

		if (
			$exec
			and ($exec['edition'] !== true) // page visu
			and ($id_table = $exec['id_table_objet'])
			and ($objet = $exec['type'])
			and ($objet === 'territoire')
			and isset($flux['args'][$id_table])
			and ($id_objet = (int) ($flux['args'][$id_table]))
		) {
			// Insertion des extras de type 'code' chargés pour ce type de territoire
			// -- acquisition de l'objet
			include_spip('action/editer_objet');
			$territoire = objet_lire('territoire', $id_objet);
			// -- tester si les extras 'code' sont chargées pour le type de territoire
			include_spip('inc/territoires_unite_peuplement');
			$extra_peuple = unite_peuplement_extra_est_charge($territoire['type'], $territoire['iso_pays'], 'code');
			if ($extra_peuple) {
				$texte .= recuperer_fond(
					'prive/objets/liste/territoires_extras',
					[
						'iso_territoire' => $territoire['iso_territoire'],
						'type_extra'     => 'code',
						'titre'          => _T('territoires:titre_liste_extras_code'),
						'pagination'     => 'page'
					]
				);
			}
		}

		// Insertion dans la fiche objet.
		if ($texte) {
			if ($p = strpos($flux['data'], '<!--affiche_milieu-->')) {
				$flux['data'] = substr_replace($flux['data'], $texte, $p, 0);
			} else {
				$flux['data'] .= $texte;
			}
		}
	}

	return $flux;
}

/**
 * Enlever l'id_territoire de la liste des critères conditionnels pour la table `spip_territoires`
 * car cela peut renvoyer une liste vide si l'env contient déjà l'id du territoire.
 *
 * @pipeline exclure_id_conditionnel
 *
 * @param array $flux Données du pipeline
 *
 * @return array Données du pipeline modifiées pour refléter le traitement.
 */
function territoires_exclure_id_conditionnel(array $flux) : array {
	if ($flux['args']['table'] == 'spip_territoires') {
		$flux['data'] = array_merge($flux['data'], ['id_territoire']);
	}

	return $flux;
}

/**
 * Complète l'action `modifier` d'un territoire avant modification:
 * - en positionnant l'indicateur d'édition à `oui`afin que les modifications manuelles soient préservées
 * lors d'un prochain rechargement.
 *
 * @pipeline pre_edition
 *
 * @param array $flux Données du pipeline fournie en entrée (chaque pipeline possède une structure de donnée propre).
 *
 * @return array Données du pipeline modifiées pour refléter le traitement.
 *
**/
function territoires_pre_edition(array $flux) : array {
	$table = $flux['args']['table'];
	$id = (int) ($flux['args']['id_objet']);
	$action = $flux['args']['action'];

	// Traitements particuliers de l'objet territoire quand celui-ci est modifié manuellement
	if (($table == 'spip_territoires') and $id) {
		// Modification d'un des champs éditables du territoire
		if ($action == 'modifier') {
			// -- On positionne l'indicateur d'édition à oui, ce qui permettra d'éviter lors
			//    d'un rechargement de perdre les modifications manuelles des champs éditables.
			$flux['data']['edite'] = 'oui';
		}
	}

	return $flux;
}

/**
 * Optimiser la base de données
 * Supprime les liens orphelins de l'objet vers quelqu'un et de quelqu'un vers l'objet.
 *
 * @pipeline optimiser_base_disparus
 *
 * @param array $flux Données du pipeline
 *
 * @return array Données du pipeline modifiées pour refléter le traitement.
 * @throws JsonException
 */
function territoires_optimiser_base_disparus(array $flux) : array {
	include_spip('action/editer_liens');
	$flux['data'] += objet_optimiser_liens(['territoire' => '*'], '*');

	return $flux;
}
